﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GanttView;
using Telerik.Windows.Controls.Scheduling;

namespace RadGanttViewQ2.WPF
{
    public class ExampleViewModel : ViewModelBase
    {
        private IDateRange visibleRange;
        private IEnumerable tasksSource;
        private TimeSpan pixelLength;
        private bool showDetails;
        private bool highlightPastTime = true;
        private bool hideWeekends;
        private IRangeGenerator specialsSlotsGenerator;
        private IRangeGenerator timeLineFilteringBehavior;

        public ExampleViewModel()
        {
            this.visibleRange = new DateRange( new DateTime( 2011, 9, 1 ), new DateTime( 2012, 12, 31 ) );
            this.pixelLength = this.CalculatePixelLength();
            this.specialsSlotsGenerator = this.highlightPastTime ? new HighlightPastRangeGenerator() : null;

            var tasks = new ObservableCollection<IGanttTask>
            {
                new GanttTask(new DateTime(2011, 9, 20), new DateTime(2011, 12, 20), "2011 Q3")
                {
                    Children =
                    {
                        new GanttTask(new DateTime(2011, 10, 20), new DateTime(2011, 10, 20), "Beta"),
                        new GanttTask(new DateTime(2011, 11, 16), new DateTime(2011, 11, 16), "Official"),
                        new GanttTask(new DateTime(2011, 12, 20), new DateTime(2011, 12, 20), "SP1"),
                        new GanttTask(new DateTime(2011, 10, 20), new DateTime(2011, 12, 25), "GanttView research") { Progress = 100 }
                    }
                },

                new GanttTask(new DateTime(2011, 12, 20),new DateTime(2012, 3, 26), "2012 Q1")
                {
                    Children =
                    {
                        new GanttTask(new DateTime(2012, 2, 15), new DateTime(2012, 2, 15), "Official"),
                        new GanttTask(new DateTime(2012, 3, 26), new DateTime(2012, 3, 26), "SP1"),
                        new GanttTask(new DateTime(2011, 12, 20), new DateTime(2012, 2, 15), "GanttView CTP")
                        {
                            Progress = 100,
                            Children =
                            {
                                new GanttTask(new DateTime(2012, 1, 10), new DateTime(2012, 2, 10), "Timeruler configurations") { Progress = 100 },
                                new GanttTask(new DateTime(2012, 1, 20), new DateTime(2012, 2, 15), "Performance improvements") { Progress = 100 },
                            }
                        },
                        new GanttTask(new DateTime(2012, 2, 15), new DateTime(2012, 3, 26), "GanttView CTP bug fixes") { Progress = 100 },
                    }
                },
                new GanttTask(new DateTime(2012, 3, 26), new DateTime(2012, 8, 10), "2012 Q2")
                {
                    Children =
                    {
                        new GanttTask(new DateTime(2012, 6, 11), new DateTime(2012, 6, 11), "Official"),
                        new GanttTask(new DateTime(2012, 8, 10), new DateTime(2012, 8, 10), "SP1"),
                        new GanttTask(new DateTime(2012, 3, 26), new DateTime(2012, 6, 11), "GanttView Beta")
                        {
                            Progress = 100,
                            Children =
                            {
                                new GanttTask(new DateTime(2012, 4, 1), new DateTime(2012, 4, 24), "Drag & drop of items") { Progress = 100 },
                                new GanttTask(new DateTime(2012, 4, 25), new DateTime(2012, 5, 5), "Filtering timeruler items") { Progress = 100 },
                                new GanttTask(new DateTime(2012, 5, 10), new DateTime(2012, 5, 25), "Highlighting of special slots") { Progress = 100 },
                            }
                        },
                        new GanttTask(new DateTime(2012, 6, 11), new DateTime(2012, 8, 10), "GanttView Beta bug fixes") { Progress = 1 },
                    }
                },
                new GanttTask(new DateTime(2012, 8, 10), new DateTime(2012, 12, 10), "2012 Q3")
                {
                    Children =
                    {
                        new GanttTask(new DateTime(2012, 10, 11), new DateTime(2012, 10, 11), "Official"),
                        new GanttTask(new DateTime(2012, 12, 10), new DateTime(2012, 12, 10), "SP1"),
                        new GanttTask(new DateTime(2012, 8, 15), new DateTime(2012, 10, 11), "GanttView final")
                        {
                            Progress = 0,
                            Children =
                            {
                                new GanttTask(new DateTime(2012, 8, 15), new DateTime(2012, 10, 11), "Mappings to business objects") { Progress = 0 },
                                new GanttTask(new DateTime(2012, 8, 15), new DateTime(2012, 10, 11), "Resource view") { Progress = 0 },
                                new GanttTask(new DateTime(2012, 8, 15), new DateTime(2012, 10, 11), "Dependencies column") { Progress = 0 },
                            }
                        },
                    }
                }
            };

            var q3_2011_tasks = (( GanttTask ) tasks[0]).Children;
            var q1_2012_tasks = (( GanttTask ) tasks[1]).Children;
            var q2_2012_tasks = (( GanttTask ) tasks[2]).Children;
            var q3_2012_tasks = (( GanttTask ) tasks[3]).Children;

            // Milestones relations.
            (( GanttTask ) q3_2011_tasks[1]).Dependencies.Add( new Dependency { FromTask = q3_2011_tasks[0], Type = RelationType.FinishStart } );
            (( GanttTask ) q3_2011_tasks[2]).Dependencies.Add( new Dependency { FromTask = q3_2011_tasks[1], Type = RelationType.FinishStart } );

            (( GanttTask ) q1_2012_tasks[0]).Dependencies.Add( new Dependency { FromTask = q3_2011_tasks[2], Type = RelationType.FinishStart } );
            (( GanttTask ) q1_2012_tasks[1]).Dependencies.Add( new Dependency { FromTask = q1_2012_tasks[0], Type = RelationType.FinishStart } );

            (( GanttTask ) q2_2012_tasks[0]).Dependencies.Add( new Dependency { FromTask = q1_2012_tasks[1], Type = RelationType.FinishStart } );
            (( GanttTask ) q2_2012_tasks[1]).Dependencies.Add( new Dependency { FromTask = q2_2012_tasks[0], Type = RelationType.FinishStart } );

            (( GanttTask ) q3_2012_tasks[0]).Dependencies.Add( new Dependency { FromTask = q2_2012_tasks[1], Type = RelationType.FinishStart } );
            (( GanttTask ) q3_2012_tasks[1]).Dependencies.Add( new Dependency { FromTask = q3_2012_tasks[0], Type = RelationType.FinishStart } );

            // Gantt stages relations.
            (( GanttTask ) q1_2012_tasks[2]).Dependencies.Add( new Dependency { FromTask = q3_2011_tasks[3], Type = RelationType.FinishStart } );
            (( GanttTask ) q2_2012_tasks[2]).Dependencies.Add( new Dependency { FromTask = q1_2012_tasks[2], Type = RelationType.FinishStart } );
            (( GanttTask ) q3_2012_tasks[2]).Dependencies.Add( new Dependency { FromTask = q2_2012_tasks[2], Type = RelationType.FinishStart } );

            this.tasksSource = tasks;
        }

        private TimeSpan CalculatePixelLength()
        {
            return this.showDetails ? TimeSpan.FromDays( 0.04 ) : TimeSpan.FromDays( 0.45 );
        }

        /// <Summary>Gets or sets VisibleRange and notfinies for changes</Summary>
        public IDateRange VisibleRange
        {
            get { return this.visibleRange; }
            set
            {
                if (this.visibleRange != value)
                {
                    this.visibleRange = value;
                    this.OnPropertyChanged( () => this.VisibleRange );
                }
            }
        }

        /// <Summary>Gets or sets TasksSource and notfinies for changes</Summary>
        public IEnumerable TasksSource
        {
            get { return this.tasksSource; }
            set
            {
                if (this.tasksSource != value)
                {
                    this.tasksSource = value;
                    this.OnPropertyChanged( () => this.TasksSource );
                }
            }
        }

        /// <Summary>Gets or sets PixelLength and notfinies for changes</Summary>
        public TimeSpan PixelLength
        {
            get { return this.pixelLength; }
            private set
            {
                if (this.pixelLength != value)
                {
                    this.pixelLength = value;
                    this.OnPropertyChanged( () => this.PixelLength );
                }
            }
        }

        /// <Summary>Gets or sets Details and notfinies for changes</Summary>
        public bool ShowDetails
        {
            get { return this.showDetails; }
            set
            {
                if (this.showDetails != value)
                {
                    this.showDetails = value;
                    this.PixelLength = this.CalculatePixelLength();
                    this.OnPropertyChanged( () => this.ShowDetails );
                }
            }
        }

        /// <Summary>Gets or sets HideWeekends and notfinies for changes</Summary>
        public bool HideWeekends
        {
            get { return this.hideWeekends; }
            set
            {
                if (this.hideWeekends != value)
                {
                    this.hideWeekends = value;
                    this.timeLineFilteringBehavior = value ? new WeekDaysGenerator { FirstDay = DayOfWeek.Monday, DaysCount = 5 } : null;
                    this.OnPropertyChanged( () => this.HideWeekends );
                    this.OnPropertyChanged( () => this.TimeLineFilteringBehavior );
                }
            }
        }

        /// <Summary>Gets or sets HighlightPastTime and notfinies for changes</Summary>
        public bool HighlightPastTime
        {
            get { return this.highlightPastTime; }
            set
            {
                if (this.highlightPastTime != value)
                {
                    this.highlightPastTime = value;
                    this.specialsSlotsGenerator = value ? new HighlightPastRangeGenerator() : null;
                    this.OnPropertyChanged( () => this.HighlightPastTime );
                    this.OnPropertyChanged( () => this.SpecialsSlotsGenerator );
                }
            }
        }

        public IRangeGenerator SpecialsSlotsGenerator
        {
            get { return this.specialsSlotsGenerator; }
        }

        public IRangeGenerator TimeLineFilteringBehavior
        {
            get { return this.timeLineFilteringBehavior; }
        }

        private class HighlightPastRangeGenerator : IRangeGenerator
        {
            public IEnumerable<IDateRange> GetRanges( IDateRange visibleRange )
            {
                yield return new DateRange( visibleRange.Start, DateTime.Today );
            }
        }
    }
}